<?php
namespace Teknisa\PLA\Modules\Ciclo\Controllers;

use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Pla10100ElaboracaoCicloController extends \Zeedhi\Framework\Controller\Report {
    
    protected $cdOperador;
    protected $reportMapping = array (
		'PLA10X00' =>
		array (
			'strategy' => 'QR2',
			'parameterMapping' =>
			array (
				"NRLISTA" => "P_NRLISTA",
			)
		)
	);
    
    public function __construct($reportService, $environment, $newCode, $pla10100Service) {
        $this->reportService = $reportService;
        $this->environment = $environment;
        $this->newCode = $newCode;
        $this->pla10100Service = $pla10100Service;
    }
    
    public function atualizarCardapio(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $this->cdOperador = $this->environment->getCdOperador();
            $this->nrOrg = $this->environment->getNrOrgTrab();
            
            $row['NRLISTA'] = $this->newCode->getCodeByTableName('LISTA', $this->nrOrg);
            $row['CDFILIAL'] = ($row['ALLCDFILIAL'] === 'S') ? $this->getAllFilialOperador() : $row['CDFILIAL'];
            if(!$this->atualizarCardapioUnidades($row))
                $response->addDataSet(new DataSet('nrlista', array($row['NRLISTA'])));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function copiarComensais(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $row['CICLOINI'] = !isset($row['CICLOINI']) ? "" : $row['CICLOINI'];
            $row['CICLOFIN'] = !isset($row['CICLOFIN']) ? "" : $row['CICLOFIN'];
            $row['NRCARDCICLOINI'] = $row['CICLOINI'];
            $row['NRCARDCICLOFIN'] = $row['CICLOFIN'];
            $excecoes = $row['EXCECOES'];
            $pratos = $this->pla10100Service->getPratosCopiarComensais($row);
            
            foreach($pratos as $prato) {
                if($excecoes[$prato['NRDIASEMCICLO']-1] !== false) { // subtrai 1 para ajustar o array (onde domingo = 0) a coluna (onde domingo = 1)
                    if($excecoes[$prato['NRDIASEMCICLO']-1] == "") {
                        $row['VRPECONSCARDCICLO'] = $row['COMENSAISPRATO'];
                    } else {
                        $row['VRPECONSCARDCICLO'] = $excecoes[$prato['NRDIASEMCICLO']-1];
                    }
                }
                $row['NRCARDCICLO'] = $prato['NRCARDCICLO'];
                $this->pla10100Service->updateComensais($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function copiarPratos(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $row['CICLOINI'] = !isset($row['CICLOINI']) ? "" : $row['CICLOINI'];
            $row['CICLOFIN'] = !isset($row['CICLOFIN']) ? "" : $row['CICLOFIN'];
            $row['NRCARDCICLOINI'] = $row['CICLOINI'];
            $row['NRCARDCICLOFIN'] = $row['CICLOFIN'];
            $pratos = $this->pla10100Service->getPratosCopiar($row);
            
            foreach($pratos as $prato) {
                $row['NRCARDCICLO'] = $prato['NRCARDCICLO'];
                if($prato['TEM_PRATO'] == 1) {
                    $this->pla10100Service->updatePrato($row);
                } else {
                    $this->pla10100Service->insertPrato($row);
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deletePrato(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $this->pla10100Service->deletePrato($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function excluirCicloPeriodo(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $row['NRCARDCICLOINI'] = $row['CICLOINI'];
            $row['NRCARDCICLOFIN'] = $row['CICLOFIN'];
            $this->pla10100Service->deletePratoPeriodo($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function getCicloData(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $config = $this->pla10100Service->getCnfCardCiclo($row);
            if (count($config) == 0){
                throw new \Exception("erro_config_vazia");
            }
            $response->addDataSet(new DataSet('cnfCardCiclo', $config));
            $dias = $this->pla10100Service->getCardaDiaCiclo($row);
            if (count($dias) == 0){
                $this->pla10100Service->insertCardCiclo($row);
                $dias = $this->pla10100Service->getCardaDiaCiclo($row);
            }
            $response->addDataSet(new DataSet('cardaDiaCiclo', $dias));
            $response->addDataSet(new DataSet('pratosCiclo', $this->pla10100Service->getPratosCiclo($row)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function importarCardapioDia(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $diasCiclos = $this->pla10100Service->getDataCardapioCiclo($row);
            if($row['SOBREPORIMPORTACAO'] === "N") {
                foreach ($diasCiclos as $diaCiclo) {
                    $this->importarCardapioDiaAcrescentar($row, $diaCiclo);
                }
            } else {
                foreach ($diasCiclos as $diaCiclo) {
                    $this->importarCardapioDiaSobrepor($row, $diaCiclo);
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function importarCicloPeriodo(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $row['NRCARDCICLOINI'] = $row['CICLOINI'];
            $row['NRCARDCICLOFIN'] = $row['CICLOFIN'];
            $row['NRCARDCICLODEST'] = $row['CICLODEST'];
            $nrDias = $row['NRCARDCICLOFIN'] ? $row['NRCARDCICLOFIN'] - $row['NRCARDCICLOINI'] : 1;
            $maxNrDias = $this->pla10100Service->getMaxDiffCiclo($row);
            if($maxNrDias < $nrDias) {
                $errorMsg = "Período informado ultrapassa o último Nº. do Ciclo.";
                $response->addDataSet(new DataSet('errorMsg', array($errorMsg)));
                throw new Exception($errorMsg);
            }
            
            if($row['SOBREPORIMPORTACAO'] === 'S') {
                $modificacoes = $this->importarCicloPeriodoSobrepor($row, $nrDias);
            } else {
                $modificacoes = $this->importarCicloPeriodoAcrescentar($row, $nrDias);
            }
            $response->addDataSet(new DataSet('modificacoes', array($modificacoes)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function savePrato(Row $request, Response $response) {
        try {
            $row = $request->getRow();
            if(isset($row["isNew"]) && $row["isNew"]) {
                $this->pla10100Service->insertPrato($row);
            } else {
                $this->pla10100Service->updatePrato($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function acrescentarPratoCiclo($row, $tipoPrato) {
        $pratoOrigem = $this->getPratoImportacaoOrigem($row, $tipoPrato);
        
        if(!empty($pratoOrigem)) {
            $pratoDestino = $this->getPratoImportacaoDestino($row, $tipoPrato);
            
            if(empty($pratoDestino)) {
                $pratoOrigem['CDCARDCICLO'] = $row['CDCARDCICLODEST'];
                $pratoOrigem['CDPADCICLO'] = $row['CDPADCICLODEST'];
                $pratoOrigem['NRCARDCICLO'] = $row['NRCARDCICLODEST'];
                $pratoOrigem['NRSEQCARDCICLO'] = $tipoPrato['NRSEQCNFCICLODEST'];
                $this->pla10100Service->insertPrato($pratoOrigem);
                return true;
            }
        }
        return false;
    }
    
    private function atualizarCardapioUnidades($row) {
        $allUnidadesOk = true;
        foreach ($row['CDFILIAL'] as $cdfilial) {
            $unidade = $this->getUnidadeToUpdate($cdfilial, $row);
            $this->pla10100Service->callProcedureAtualizaCardapio($unidade);
            $nrLogCardCiclo = $this->pla10100Service->getLogErrors($unidade);
            if(!empty($nrLogCardCiclo)) {
                $allUnidadesOk = false;
                $this->pla10100Service->insertLogIntoLista($unidade);
            }
        }
        return $allUnidadesOk;
    }
    
    private function buildPratoToImport($row, $diaCiclo, $tipoPratoSubs) {
        $prato = array(
            "NRSEQCARDCICLO" => $tipoPratoSubs['NRSEQCNFCICLO'],
            "DTCARDAPIO" => substr($diaCiclo['DTCARDAPIO'], 0, 10),
            "CDCARDCICLO" => $row['CDCARDCICLO'],
            "CDTIPOPRAT" => $tipoPratoSubs['CDTIPOPRAT'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "NRCARDCICLO" => $diaCiclo['NRCARDCICLO'],
            "NRSEQPRATCAR" => $tipoPratoSubs['NRORDCNFMES'],
            "CDPADCICLO" => $row['CDPADCICLO'],
            "CDTIPOPRATCAR" => $tipoPratoSubs['CDTIPOPRATCAR'],
            "CDFILIAL" => $row['CDFILIAL']
        );
        return $prato;
    }
    
    private function getAllFilialOperador() {
        $conditions = array(
            array(
                "columnName" => "CDOPERADOR",
                "operator" => "=",
                "value" => $this->cdOperador
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->nrOrg
            )
        );
        return $this->pla10100Service->getAllFilialOperador($conditions);
    }
    
    private function getPratoImportacaoDestino($row, $tipoPrato) {
        $row['CDCARDCICLO'] = $row['CDCARDCICLODEST'];
        $row['CDPADCICLO'] = $row['CDPADCICLODEST'];
        $row['CDTIPOPRAT'] = $tipoPrato['CDTIPOPRAT'];
        $row['NRSEQCARDCICLO'] = $tipoPrato['NRSEQCNFCICLODEST'];
        $row['NRCARDCICLO'] = $row['NRCARDCICLODEST'];
        $pratoDestino = $this->pla10100Service->getPratoImportacao($row);
        if(!empty($pratoDestino))
            return $pratoDestino[0];
            
        return array();
    }
    
    private function getPratoImportacaoOrigem($row, $tipoPrato) {
        $row['CDCARDCICLO'] = $row['CDCARDCICLOORIG'];
        $row['CDPADCICLO'] = $row['CDPADCICLOORIG'];
        $row['CDTIPOPRAT'] = $tipoPrato['CDTIPOPRAT'];
        $row['NRSEQCARDCICLO'] = $tipoPrato['NRSEQCNFCICLO'];
        $row['NRCARDCICLO'] = $row['NRCARDCICLOORIG'];
        $pratoOrigem = $this->pla10100Service->getPratoImportacao($row);
        if(!empty($pratoOrigem))
            return $pratoOrigem[0];
            
        return array();
    }
    
    private function getUnidadeToUpdate($cdfilial, $row) {
        $unidade = array(
            "CDFILIAL" => $cdfilial,
            "CDOPERADOR" => $this->cdOperador,
            "DTINICICLO" => $row['DTINICICLO'],
            "NRLISTA" => $row['NRLISTA'],
            "NRLOGCARDCICLO" => 0,
            "CDPADCICLO" => null,
            "CDSERVFILI" => null
        );
        $paramsUnidade = $this->pla10100Service->getPadraoCicloServicoPorUnidade($unidade);
        $unidade['CDPADCICLO'] = isset($paramsUnidade[0]['CDPADCICLO']) ? $paramsUnidade[0]['CDPADCICLO'] : null;
        $unidade['CDSERVFILI'] = isset($paramsUnidade[0]['CDSERVFILI']) ? $paramsUnidade[0]['CDSERVFILI'] : null;
        
        $nrLogCardCiclo = $this->pla10100Service->getNrLogCardCiclo($unidade)[0]['NRLOGCARDCICLO'];
        $nrLogCardCiclo++;
        $unidade['NRLOGCARDCICLO'] = str_pad($nrLogCardCiclo, 10, '0', STR_PAD_LEFT);
        $unidade['CDLISTA'] = $unidade['CDFILIAL'].$unidade['NRLOGCARDCICLO'];
        
        return $unidade;
    }
    
    private function importarCardapioDiaAcrescentar($row, $diaCiclo) {
        foreach ($row['tiposPratoSubstituicao'] as $tipoPratoSubs) {
            $tipoPratoDia = array(
                "CDCARDCICLO" => $row['CDCARDCICLO'],
                "CDPADCICLO" => $row['CDPADCICLO'],
                "NRCARDCICLO" => $diaCiclo['NRCARDCICLO'],
                "CDTIPOPRAT" => $tipoPratoSubs['CDTIPOPRAT'],
                "NRSEQCARDCICLO" => $tipoPratoSubs['NRSEQCNFCICLO']
            );
            $pratoCiclo = $this->pla10100Service->getPratoCicloCardapioDia($tipoPratoDia);
            if(!$pratoCiclo) {
                $pratoToSave = $this->buildPratoToImport($row, $diaCiclo, $tipoPratoSubs);
                $this->pla10100Service->insertPratoFromCardapio($pratoToSave);
            }
        }
    }
    
    private function importarCardapioDiaSobrepor($row, $diaCiclo) {
        foreach ($row['tiposPratoSubstituicao'] as $tipoPratoSubs) {
            $tipoPratoDia = array(
                "CDCARDCICLO" => $row['CDCARDCICLO'],
                "CDPADCICLO" => $row['CDPADCICLO'],
                "NRCARDCICLO" => $diaCiclo['NRCARDCICLO'],
                "CDTIPOPRAT" => $tipoPratoSubs['CDTIPOPRAT'],
                "NRSEQCARDCICLO" => $tipoPratoSubs['NRSEQCNFCICLO']
            );
            $pratoCiclo = $this->pla10100Service->getPratoCicloCardapioDia($tipoPratoDia);
            if($pratoCiclo) {
                $this->pla10100Service->deletePrato($tipoPratoDia);
            }
            $pratoToSave = $this->buildPratoToImport($row, $diaCiclo, $tipoPratoSubs);
            $this->pla10100Service->insertPratoFromCardapio($pratoToSave);
        }
    }
    
    private function importarCicloPeriodoAcrescentar($row, $nrDias) {
        $modificacoes = 0;
        $tiposPratos = $this->pla10100Service->getTipoPratoCorrespondente($row);
        for($i = 0; $i <= $nrDias; $i++) {
            $row['NRCARDCICLOORIG'] = $row['NRCARDCICLOINI'] + $i;
            foreach ($tiposPratos as $tipoPrato) {
                if($this->acrescentarPratoCiclo($row, $tipoPrato))
                    $modificacoes++;
            }
            $row['NRCARDCICLODEST']++;
        }
        return $modificacoes;
    }
    
    private function importarCicloPeriodoSobrepor($row, $nrDias) {
        $modificacoes = 0;
        $tiposPratos = $this->pla10100Service->getTipoPratoCorrespondente($row);
        for($i = 0; $i <= $nrDias; $i++) {
            $row['NRCARDCICLOORIG'] = $row['NRCARDCICLOINI'] + $i;
            foreach ($tiposPratos as $tipoPrato) {
                if($this->sobreporPratoCiclo($row, $tipoPrato))
                    $modificacoes++;
            }
            $row['NRCARDCICLODEST']++;
        }
        return $modificacoes;
    }
    
    private function sobreporPratoCiclo($row, $tipoPrato) {
        $pratoOrigem = $this->getPratoImportacaoOrigem($row, $tipoPrato);
        if(!empty($pratoOrigem)) {
            $pratoDestino = $this->getPratoImportacaoDestino($row, $tipoPrato);
            
            $pratoOrigem['CDCARDCICLO'] = $row['CDCARDCICLODEST'];
            $pratoOrigem['CDPADCICLO'] = $row['CDPADCICLODEST'];
            $pratoOrigem['NRCARDCICLO'] = $row['NRCARDCICLODEST'];
            $pratoOrigem['NRSEQCARDCICLO'] = $tipoPrato['NRSEQCNFCICLODEST'];
            if(empty($pratoDestino)) {
                $this->pla10100Service->insertPrato($pratoOrigem);
            } else {
                $this->pla10100Service->updatePrato($pratoOrigem);
            }
            return true;
        }
        return false;
    }
}