<?php
namespace Teknisa\PLA\Modules\Medicao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

class Pla02000RetiradaPlanejamentoController extends \Teknisa\GER\Controllers\TeknisaCrud {

	const PREVRETI = 'pla02000_prevretiexistente';
	const RETIRADA = 'pla02000_retiradaexistente';
    const PARAMETROS = 'pla02000_retiradaplanejamentoparametrosbloqueio';

    protected $retiradaPlanejamentoService;
    protected $dataSourceManager;
    protected $dataSourceName = "pla02000_retiradaplanejamento";
    protected $environment;
    protected $libUtilities;
    protected $parameterBagColumns = array(
    	"NRORG",
		"CDFILIAL",
		"CDSERVFILI",
		"DTRETIRADA",
		"DTCARDAPIO",
		"NRREQUEXTRRE"
	);

	 public function __construct($crudService, $parameterBag, $dataSourceManager, $retiradaPlanejamentoService, $authService, $environment,$libUtilities) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
    	$this->dataSourceManager = $dataSourceManager;
        $this->retiradaPlanejamentoService = $retiradaPlanejamentoService;
        $this->authService = $authService;
        $this->environment = $environment;
        $this->libUtilities=$libUtilities;
    }

	public function salvaRetirada(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $cdservfili = $row['CDSERVFILI'];
            $dtretirada = $row['DTRETIRADA'];
            $cdtpretipadr = $row['CDTPRETIPADR'];
            $cdfilial_logada = $this->environment->getCdfilial();
            
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            
            $this->setParamsEstoq($row); 
            $retirada = self::retornaRetirada($row);
            if (!empty($retirada)) {
                throw new \Exception("Já existe retirada cadastrada.");
            }

            $validaDataRetirada = $this->validaDataRetirada($row);

            if ($validaDataRetirada) {
                throw new \Exception($validaDataRetirada);
            }

            if (!$cdtpretipadr) {
                throw new \Exception("Não existe tipo de retirada parametrizado.");
            }
            $row['DTPREVRETI'] = $row['DTCARDAPIO'];
            $prevreti = $this->retornaPrevreti($row);

            if($prevreti) {
            	$idretiprat = $prevreti[0]['IDPREVTPPRAT'];
            	$nrcardapio = $this->retiradaPlanejamentoService->retornaNrcardapio($row);
            	if(!$nrcardapio) {
            		$nrcardapio = $this->retiradaPlanejamentoService->insereCardapio($row);
            	}
            	$row['IDTPLANCTO'] = 3;
            	$row['DTLANCESTQ'] = $dtretirada;
            	$row['DSLANCESTQ'] = "Retirada para o cardápio: ".$dtretirada.", Servico: ".$cdservfili;
            	$row['NRCARDAPIO'] = $nrcardapio;
                $row['IDUTRETIPRAT'] = $idretiprat;
            	
            	$nrlancto = $this->retiradaPlanejamentoService->insereLancamento($row);
                $row['NRLANCESTQ'] = $nrlancto;
                
            	$nrretirada = $this->retiradaPlanejamentoService->insereRetirada($row);

                $index = 0;
            	foreach ($prevreti as $item) {
                    $seq = str_pad($index++, 4, "0", STR_PAD_LEFT);
                    $item['NRLANCESTQ'] = $nrlancto;
                    $item['NRSEQUITEM'] = $seq;
                    $item['QTRETIRADA'] = $item['QTPREVLIB'];
                    $item['IDTIPOMOVI'] = '3';
                    $item['DTRETIRADA'] = $row['DTRETIRADA'];
                    $item['IDORDEMCLC'] = 9;
                    $item['CDFILIAL'] = $row['CDFILIAL'];
                    $item['IDEVOLESTOQ'] = 'N';
                    $item['CDTIPORETI'] = $row['CDTPRETIPADR'];
                    $item['NRRETIRADA'] = $nrretirada;
                    $item['NRCARDAPIO'] = $nrcardapio;
                    $item['IDINTESIST'] = 'N';
                    $item['QTPREVISRETI'] = $item['QTPREVRETI'];
                    $item['QTDEVOLVRETI'] = $item['QTPREVDEVOL'];
                    $item['QTEXTRARETI'] = $item['QTPREVEXTRA'];
                    $this->setParamsEstoq($item);
         			$this->retiradaPlanejamentoService->insereItemLancto($item, $cdfilial_logada);
            		$this->retiradaPlanejamentoService->insereItemreti(	$item, $cdfilial_logada );
            	}
            	$response->addMessage(new Message(true));
            } else {
            	$response->addMessage(new Message(false));
            }

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function setParamsEstoq(&$row){
        $row['CDALMOXARIFE'] = isset($row['CDALMOXARIFE']) && $row['CDALMOXARIFE'] ? $row['CDALMOXARIFE'] : ' '; 
        $row['CDLOCALESTOQ'] = isset($row['CDLOCALESTOQ']) && $row['CDLOCALESTOQ'] ? $row['CDLOCALESTOQ'] : ' '; 
        $row['NRLOTEESTQ']   = isset($row['NRLOTEESTQ'])   && $row['NRLOTEESTQ']   ? $row['NRLOTEESTQ']   : ' '; 
        $row['NRSUBLOTE']    = isset($row['NRSUBLOTE'])    && $row['NRSUBLOTE']    ? $row['NRSUBLOTE']    : ' ';   
    }

    public function salvaNovaRetirada(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $dtretirada = $row['DTRETIRADA'];
            
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            
            $this->setParamsEstoq($row);          
            $dtposicao = $this->retiradaPlanejamentoService->getDataDeString($dtretirada);
            $dtposicao->add(new \DateInterval('P1D'));
            $dtposicao = $dtposicao->format('d/m/Y');
            $cdfilial_logada = $this->environment->getCdfilial();
            $row['CDFILILANC'] = null;
            $row['NRLANCESTQ'] = null;
            $row['NRSEQUITEM'] = '0000';
            $row['CDPRODMOVI'] = null;
            $row['DTPOSICAO'] = $dtposicao;
            $this->retiradaPlanejamentoService->iniciaConexao();  
            try {
                $error = array();
                $nrcardapio = $this->retiradaPlanejamentoService->retornaNrcardapio($row);
                if(!$nrcardapio) {
                    $nrcardapio = $this->retiradaPlanejamentoService->insereCardapio($row);
                }
                $row['IDTPLANCTO'] = 3;
                $row['DTLANCESTQ'] = $dtretirada;
                $row['DSLANCESTQ'] = "Retirada para o cardápio: ".$dtretirada.", Servico: ".$row['CDSERVFILI'];
                
                $nrlancto = $this->retiradaPlanejamentoService->insereLancamento($row);
                $row['NRCARDAPIO'] = $nrcardapio;
                $row['NRLANCESTQ'] = $nrlancto;
                $nrretirada = $this->retiradaPlanejamentoService->insereRetirada($row);
                
                $row['IDTIPOMOVI'] = '3';
                $row['IDORDEMCLC'] = 9;
                $row['IDEVOLESTOQ'] = 'N';
                $row['NRLANCESTQ'] = $nrlancto;
                $row['NRRETIRADA'] = $nrretirada;
                $row['NRCARDAPIO'] = $nrcardapio;
                $row['IDINTESIST'] = 'N';
                
                $isValidStock = $this->retiradaPlanejamentoService->isValidStock($row, $error);
                if(!$isValidStock) {
                    $response->addDataSet(new DataSet('error', $error));
                    throw new \Exception("INVALID_STOCK");
                }
                
                $this->retiradaPlanejamentoService->insereItemLancto($row, $cdfilial_logada);
                $this->retiradaPlanejamentoService->insereItemreti($row, $cdfilial_logada);
                $this->retiradaPlanejamentoService->realizaCommit();
            } catch(\Exception $e){
                $this->retiradaPlanejamentoService->realizaRollback();
                throw $e;
            }
            $this->retiradaPlanejamentoService->atualizarEstoque($this->environment->getCdfilial(), $nrlancto); // passava idevolestoq como 'S'
        } catch (\Exception $e) {
            $this->treatError($e, $response);
        }
    }
    
    private function isExceptionPebbian(\Exception $error){
        return $error->getCode() === $this->retiradaPlanejamentoService->getCodErrorPebbian();
    }

    private function treatError(\Exception $e, $response){
        $message = $e->getMessage();
        $code = $e->getCode();
        $arrayMessage = explode("\n", $message);
        if($this->isExceptionPebbian($e)){
            $response->setError(new DTO\Response\Error($message, $code));
            return;
        }
        if(!empty($arrayMessage)){
            $clean_message = "";
            foreach($arrayMessage as $currentString){
                if(stripos($currentString, "ORA-20501") !== false){
                    $clean_message = str_replace("ORA-20501: ", "", $currentString);
                }
            }
            if(!empty($clean_message)) {
                $response->setError(new DTO\Response\Error(utf8_decode($clean_message), 1)); 
            } else {
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));
            }
        } else {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function salvaNovoPratoProduto(DTO\Request\Row $request, DTO\Response $response) {

        try {
            $row = $request->getRow();
            
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            
            $cdfilimovi = $row['CDFILIAL'];
            $nrlancestq = $row['NRLANCESTQ'];
            $dtretirada = $row['DTRETIRADA'];
            
            $filialLancamento = $this->retiradaPlanejamentoService->retornaFilialLancamentoRetirada($cdfilimovi, $row['NRCARDAPIO']);
            $dtposicao = $this->retiradaPlanejamentoService->getDataDeString($dtretirada);
            $dtposicao->add(new \DateInterval('P1D'));
            $dtposicao = $dtposicao->format('d/m/Y');
            $row['NRSEQUITRETI'] = ' ';
            $itemreti = $this->retiradaPlanejamentoService->Vd_ExistProdAlmox($row);

            if($itemreti) {
                throw new \Exception("Já existe item cadastrado.");
            }

            $maiorseq = $this->retiradaPlanejamentoService->retornaMaxSequencial($row);
            $seq = str_pad(($maiorseq+1), 4, "0", STR_PAD_LEFT);

            $row['CDFILILANC'] = $filialLancamento;
            $row['NRSEQUITEM'] = $seq;
            $row['CDPRODMOVI'] = null;
            $row['DTPOSICAO'] = $dtposicao;
            $this->setParamsEstoq($row);
            $quantidadeInvalida = $this->retiradaPlanejamentoService->validaQuantidadeEstoque($row);

            if($quantidadeInvalida) {
                throw new \Exception($quantidadeInvalida);
            }

            $controleEstoque = $this->verificaControleEstoque($row);

            if (!$controleEstoque) {
                throw new \Exception("Produto informado não controla estoque.");
            }
            
            $row['IDTIPOMOVI'] = '3';
            $row['IDORDEMCLC'] = 9;
            $row['IDEVOLESTOQ'] = 'N';
            $row['IDINTESIST'] = 'N';
            $this->retiradaPlanejamentoService->insereItemLancto($row, $filialLancamento);
            $this->retiradaPlanejamentoService->insereItemreti($row, $filialLancamento);
            $this->retiradaPlanejamentoService->atualizarEstoque($filialLancamento, $nrlancestq);// passava idevolestoq como 'S'

        } catch (\Exception $e) {
            $this->treatError($e, $response);
        }
    }

    private function verificaControleEstoque($row) {
        $conditions = array (
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
                "columnName" => "CDPRODUTO",
                "operator" => "=",
                "value" => $row['CDPRODUTO']
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('prodfili', $conditions);
        $rows = $dataSet->getRows();

        if ($rows[0]['IDCNTRESTOQ'] == 'N')
            return false;
        return true;
    }
    
    public function salvaAlteracaoProdutoInline(DTO\Request $request, DTO\Response $response) {
        $this->retiradaPlanejamentoService->iniciaConexao();
        try{
            $rows = $request->getDataSet()->getRows();
            //coleta de dados
            $this->libUtilities->saveCollectedData($rows[0]['CDFILIAL'],'RETIRADA');
            $errosParaRetornar = [];
            $cdfilancestq = null;
            $nrlancestq = null;
            foreach ($rows as $row) {
                if ($cdfilancestq == null){
                    $cdfilancestq = $row['CDFIITLANEST'];
                    if (empty($cdfilancestq)) {
                        $cdfilancestq = $this->retiradaPlanejamentoService->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
                    }
                    $nrlancestq = $row['NRLANCESTQ'];
                }
                $this->salvaAlteracaoProduto(new DTO\Request\Row($row, "", "", ""), $response, $errosParaRetornar);
            }
            if(count($errosParaRetornar) > 0){
                $response->addDataSet(new DataSet('produtos', $errosParaRetornar));
                $response->setError(new DTO\Response\Error('SALDO_INDISPONIVEL', 1));
                $this->retiradaPlanejamentoService->realizaRollback();
                return;
            }
            $this->retiradaPlanejamentoService->atualizarEstoque($cdfilancestq, $nrlancestq);
            $this->retiradaPlanejamentoService->realizaCommit();
        } catch (\Exception $e) {
            $this->retiradaPlanejamentoService->realizaRollback();
            $this->treatError($e, $response);
        }
    }
    
    public function salvaAlteracaoProduto(DTO\Request\Row $request, DTO\Response $response, &$errosParaRetornar = null) {
        try {
            $row = $request->getRow();
            
               //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            
            $cdfilimovi = $row['CDFILIAL'];
            $nrlancestq = $row['NRLANCESTQ'];

            $error = array();
            $isValidStock = true;
            if(isset($row['NRSEQUITRETI'])){
                $isValidStock = $this->retiradaPlanejamentoService->isValidStock($row, $error, $row["NRRETIRADA"], $row["NRCARDAPIO"], $row["NRSEQUITRETI"]);
            }
            else
                $isValidStock = $this->retiradaPlanejamentoService->isValidStock($row, $error);
            if(!$isValidStock) {
                if($errosParaRetornar === null){
                    $response->addDataSet(new DataSet('error', $error));
                    throw new \Exception("INVALID_STOCK");
                }
                else {
                    array_push($errosParaRetornar, $row["CDARVPROD"] . " " . $row["NMPRODUTO"] . "&&" . $error[0]);
                }
            }
            $prodJaCad = $this->retiradaPlanejamentoService->Vd_ExistProdAlmox($row);
            if($prodJaCad){
                if($errosParaRetornar === null){
                    throw new \Exception("prodJaCadastradoParaRetiradaAtual");
                }
                else {
                    array_push($errosParaRetornar, $row["CDARVPROD"] . " " . $row["NMPRODUTO"] . "&&" . "prodJaCadastradoParaRetiradaAtual");
                }
            }
            $cdfilancestq = $this->retiradaPlanejamentoService->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
            $itlanctoest = $this->retiradaPlanejamentoService->retornaItlanctoest($cdfilancestq, $nrlancestq, $row['NRSEQUITRETI']);
            $row['CDFILANCESTQ'] = $cdfilancestq;
            if($row['QTRETIRADA'] > 0) {
                if($itlanctoest) {
                    $rowItemLancto = clone $row;
                    $rowItemLancto['CDFILIAL'] = $cdfilancestq;
                    $this->retiradaPlanejamentoService->alteraItemLancto($rowItemLancto);
                } else {
                    $row['NRSEQUITEM'] = $row['NRSEQUITRETI'];
                    $row['IDTIPOMOVI'] = '3';
                    $row['IDORDEMCLC'] = 9;
                    $row['IDEVOLESTOQ'] = 'N';
                    $this->setParamsEstoq($row);
                    $this->retiradaPlanejamentoService->insereItemLancto($row, $cdfilancestq);
                }
            } else {
                if($itlanctoest) {
                    $this->retiradaPlanejamentoService->removeRelacaoItemRetiItlanctoest($row);
                    $this->retiradaPlanejamentoService->excluiItemLancto($row);
                }
            }
            $this->retiradaPlanejamentoService->alteraItemreti($row);
            if($errosParaRetornar === null){
                $this->retiradaPlanejamentoService->atualizarEstoque($cdfilancestq, $nrlancestq);// passava idevolestoq como 'S'
            }

        } catch (\Exception $e) {
            if($errosParaRetornar != null){
                throw $e;
            }
            $this->treatError($e, $response);
        }
    }

    public function excluiProdutos(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->retiradaPlanejamentoService->iniciaConexao();
        try {
            $itens = $request->getDataSet()->getRows();
            $firstItem = null;
            if(count($itens) > 0){
                $firstItem = $itens[0];
            }
            $cdfilancestq = $this->retiradaPlanejamentoService->retornaFilialLancamentoRetirada($firstItem['CDFILIAL'], $firstItem['NRCARDAPIO']);
            foreach ($itens as $item) {
                $this->retiradaPlanejamentoService->excluiItemreti($item);
                if (isset($item['NRLANCESTQ']) && $item['NRLANCESTQ']) {
                    $item['CDFILANCESTQ'] = $cdfilancestq;
                    $this->retiradaPlanejamentoService->excluiItemLancto($item);
                }
            }
            if($firstItem){
                $lancamento = $this->retiradaPlanejamentoService->dadosLancamento($firstItem['CDFILIAL'], $firstItem['NRCARDAPIO'], $firstItem['NRRETIRADA']);
                if(!empty($lancamento['CDFILANCESTQ'])){
                    $this->retiradaPlanejamentoService->atualizarEstoque($lancamento['CDFILANCESTQ'], $lancamento['NRLANCESTQ']);
                }
            }
            $this->retiradaPlanejamentoService->realizaCommit();
        } catch (\Exception $e) {
            $this->retiradaPlanejamentoService->realizaRollback();
            $this->treatError($e, $response);
        }
    }

    public function excluiProdutosPorPrato(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->retiradaPlanejamentoService->iniciaConexao();
            $pratos = $request->getDataSet()->getRows();
            $nrlancestq = '';
            $cdfiitlanest = null;
            foreach ($pratos as $prato) {
                $itens = $this->retiradaPlanejamentoService->retornaItensPorPrato($prato);
                foreach ($itens as $item) {
                    $nrlancestq = $item->getNrlancestq();
                    $cdfiitlanest = $item->getCdfiitlanest();
                    $nrsequitreti = $item->getNrsequitreti();
                    $prato['NRSEQUITRETI'] = $nrsequitreti;
                    $this->retiradaPlanejamentoService->excluiItemreti($prato);

                    if($nrlancestq) {
                        $rowItem = array(
                            'CDFILANCESTQ' => $item->getCdfiitlanest(),
                            'NRLANCESTQ' => $nrlancestq,
                            'NRSEQUITRETI' => $nrsequitreti
                        );
                        $this->retiradaPlanejamentoService->excluiItemLancto($rowItem);
                    }
                }
            }
            if($nrlancestq && $cdfiitlanest){
                $this->retiradaPlanejamentoService->atualizarEstoque($cdfiitlanest, $nrlancestq); // passava idevolestoq como 'S'
            }
            $this->retiradaPlanejamentoService->realizaCommit();
        } catch (\Exception $e) {
            $this->retiradaPlanejamentoService->realizaRollback();
            $this->treatError($e, $response);
        }
    }

    public function atualizarEstoque(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $cdfilancestq = $this->retiradaPlanejamentoService->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
            $this->retiradaPlanejamentoService->atualizarEstoque($cdfilancestq, $row['NRLANCESTQ']);// passava idevolestoq como 'S'
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function excluiRetirada(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->retiradaPlanejamentoService->excluiLancamentoRetirada($row);
        } catch (\Exception $e) {
            $this->treatError($e, $response);
        }
    }

    private function retornaRetirada($row) {
        $nrcardapio = $this->retiradaPlanejamentoService->retornaNrcardapio($row);
        return $this->retiradaPlanejamentoService->retornaRetiradasExistentes($row['CDFILIAL'], $nrcardapio);
    }

    public function retornaPrevreti($row) {
		$filterCriteria = new FilterCriteria(self::PREVRETI);
		$this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDSERVFILI', 'DTPREVRETI', 'NRREQUEXTRRE');
		$filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
		$filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
		$filterCriteria->addCondition('CDSERVFILI', $row['CDSERVFILI']);
		$filterCriteria->addCondition('DTPREVRETI', $row['DTPREVRETI']);
		$filterCriteria->addCondition('NRREQUEXTRRE', $row['NRREQUEXTRRE']);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
    	$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
    	$rows = $dataSet->getRows();

    	return $rows;
    }

    public function retornaParametrosBloqueio(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $cdfilial = $row['CDFILIAL'];

        $parametrosBloqueio = $this->getParametrosBloqueio($cdfilial);
        $response->addDataSet($parametrosBloqueio);
    }

    private function getParametrosBloqueio($cdfilial) {
        $filterCriteria = new FilterCriteria(self::PARAMETROS);
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }

    public function validaDataRetirada($row) {
        $mensagem = false;
        $parametrosBloqueio = $this->getParametrosBloqueio($row['CDFILIAL'])->getRows();
      
        if (!empty($parametrosBloqueio)) {
            $dtprocessa = \DateTime::createFromFormat('d/m/Y H:i:s', $parametrosBloqueio[0]['DTPROCESSA']);
            $dtretirada = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTRETIRADA']."00:00:00");
            $idlancdtmen = $parametrosBloqueio[0]['IDLANCDTMEN'];
            $idlancdtmai = $parametrosBloqueio[0]['IDLANCDTMAI'];
            $nrdiassaid = $parametrosBloqueio[0]['NRDIASSAID'] ? $parametrosBloqueio[0]['NRDIASSAID'] : 0;
            $nrdiassaidpd = $parametrosBloqueio[0]['NRDIASSAIDPD'] ? $parametrosBloqueio[0]['NRDIASSAIDPD'] : 0;
            $idlancdtmenfil = $parametrosBloqueio[0]['IDLANCDTMENFIL'];
            $nrdiassaidfil = $parametrosBloqueio[0]['NRDIASSAIDFIL'] ? $parametrosBloqueio[0]['NRDIASSAIDFIL'] : 0;
            $idlancdtmaifil = $parametrosBloqueio[0]['IDLANCDTMAIFIL'];
            $nrdiassaidpdfil = $parametrosBloqueio[0]['NRDIASSAIDPDFIL'] ? $parametrosBloqueio[0]['NRDIASSAIDPDFIL'] : 0;
            $dateProcessa = \DateTime::createFromFormat('d/m/Y', explode(" ",$parametrosBloqueio[0]['DTPROCESSA'])[0]);
            $dateRetirada = \DateTime::createFromFormat('d/m/Y', $row['DTRETIRADA']);
            $dtprocessaformat = strtotime($dateProcessa->format('Y-m-d'));
            $dtretiradaformat = strtotime($dateRetirada->format('Y-m-d'));
            if ($dtretiradaformat < $dtprocessaformat) {
                if ($idlancdtmenfil == 'N') {
                    if ($idlancdtmen == 'N') {
                        $mensagem = "Data informada não deve ser menor que a data de processamento.";
                    } else {
                        if( $dtretirada < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaid.'D')))) {
                            $mensagem = "Data informada fora do período permitido.";
                        }
                    }
                } else {
                    if( $dtretirada < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaidfil.'D')))) {
                        $mensagem = "Data informada fora do período permitido.";
                    }
                }
            } else if ($dtretiradaformat > $dtprocessaformat) {
                if ($idlancdtmaifil == 'S') {
                    if( $dtretirada > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpdfil.'D')))) {
                        $mensagem = "Data informada fora do período permitido.";
                    }
                } else {
                    if ($idlancdtmai == 'S') {
                        if ( $dtretirada > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpd.'D')))) {
                            $mensagem = "Data informada fora do período permitido.";
                        }
                    } else {
                        $mensagem = "Data informada não deve ser maior que a data de processamento.";
                    }
                }
            }
        }
        return $mensagem;
    }

    public function autenticaOperador(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            $email = $row['email'];
            $senha = $row['senha'];
            $cdgroperreti = $row['cdgroperreti'];
            $operador = $this->authService->autenticar($email, $senha);
            $conditions = array (
                array (
                    "columnName" => "CDOPERADOR",
                    "operator" => "=",
                    "value" => $operador["cdoperador"]
                ),
                array (
                    "columnName" => "CDGRUPOPER",
                    "operator" => "=",
                    "value" => $cdgroperreti
                )
            );
            $dataSet = $this->crudService->getFilteredDataSet('opergrupop', $conditions);
            $rows = $dataSet->getRows();
            if (empty($rows)) {
                throw new \Exception("Operador não pertence ao grupo responsável pela autorização.");
            } else {
                return true;
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        } 

    }
    
    public function verificaParamExibeSaldo (DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $cdfilial = $row['CDFILIAL'];
        $dataSet = $this->retiradaPlanejamentoService->verificaParamExibeSaldo($cdfilial);
        $response->addDataSet(new DataSet("data", $dataSet));
    }
    
    public function atualizaCustoTotal(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $custoTotalRow = $this->retiradaPlanejamentoService->getCustoTotal($row['CDFILIAL'], $row['NRRETIRADA'], $row['NRCARDAPIO']);
        if(!$custoTotalRow){
            $response->addDataSet(new DataSet("custototal", [0]));
        }
        else{
            $vrcustototal = $custoTotalRow['CUSTOTOTAL'];
            if($vrcustototal == null)
                $vrcustototal = 0;
            $response->addDataSet(new DataSet("custototal", [$vrcustototal]));
        }
    }

}